# $Id: mqtt.py 1130 2014-11-24 13:13:36Z mwall $
# Copyright 2013 Matthew Wall
"""
Upload data to MQTT server

This service requires the python bindings for mqtt:

   pip install paho-mqtt

Minimal configuration:

[StdRestful]
    [[MQTT]]
        server_url = mqtt://username:password@localhost:1883/
        topic = weather
        units = METRIC
"""

import Queue
import paho.mqtt.client as mqtt
import sys
import syslog
import urlparse

import weewx
import weewx.restx
import weewx.units
from weeutil.weeutil import accumulateLeaves

VERSION = "0.4rc2"

if weewx.__version__ < "3":
    raise weewx.UnsupportedFeature("weewx 3 is required, found %s" %
                                   weewx.__version__)

def logmsg(level, msg):
    syslog.syslog(level, 'restx: MQTT: %s' % msg)

def logdbg(msg):
    logmsg(syslog.LOG_DEBUG, msg)

def loginf(msg):
    logmsg(syslog.LOG_INFO, msg)

def logerr(msg):
    logmsg(syslog.LOG_ERR, msg)

def _compat(d, old_label, new_label):
    if d.has_key(old_label) and not d.has_key(new_label):
        d.setdefault(new_label, d[old_label])
        d.pop(old_label)

class MQTT(weewx.restx.StdRESTbase):
    def __init__(self, engine, config_dict):
        """This service recognizes standard restful options plus the following:

        Required parameters:

        server_url: URL of the broker, e.g., something of the form
          mqtt://username:password@localhost:1883/
        Default is None

        Optional parameters:

        unit_system: one of US, METRIC, or METRICWX
        Default is None; units will be those of data in the database

        topic: the MQTT topic under which to post
        Default is 'weather'
        """
        super(MQTT, self).__init__(engine, config_dict)        
        loginf("service version is %s" % VERSION)
        try:
            site_dict = config_dict['StdRESTful']['MQTT']
            site_dict = accumulateLeaves(site_dict, max_level=1)
            site_dict['server_url']
        except KeyError, e:
            logerr("Data will not be posted: Missing option %s" % e)
            return

        # for backward compatibility: 'units' is now 'unit_system'
        _compat(site_dict, 'units', 'unit_system')

        if not site_dict.has_key('topic'):
            site_dict.setdefault('topic', 'weather')
        site_dict['manager_dict'] = weewx.manager.get_manager_dict(
            config_dict['DataBindings'], config_dict['Databases'], 'wx_binding')

        self.archive_queue = Queue.Queue()
        self.archive_thread = MQTTThread(self.archive_queue, **site_dict)
        self.archive_thread.start()
        self.bind(weewx.NEW_ARCHIVE_RECORD, self.new_archive_record)

        if 'topic' in site_dict:
            loginf("Topic is %s" % site_dict['topic'])
        if 'unit_system' in site_dict:
            loginf("Desired unit system is %s" % site_dict['unit_system'])
        loginf("Data will be posted to %s" % site_dict['server_url'])

    def new_archive_record(self, event):
        self.archive_queue.put(event.record)

class MQTTThread(weewx.restx.RESTThread):
    def __init__(self, queue, manager_dict, server_url,
                 topic='', unit_system=None,
                 post_interval=None, max_backlog=sys.maxint, stale=None,
                 log_success=True, log_failure=True,
                 timeout=60, max_tries=3, retry_wait=5):
        super(MQTTThread, self).__init__(queue,
                                         protocol_name='MQTT',
                                         manager_dict=manager_dict,
                                         post_interval=post_interval,
                                         max_backlog=max_backlog,
                                         stale=stale,
                                         log_success=log_success,
                                         log_failure=log_failure,
                                         max_tries=max_tries,
                                         timeout=timeout,
                                         retry_wait=retry_wait)
        self.server_url = server_url
        self.topic = topic
        self.units = weewx.units.unit_constants[unit_system] if unit_system is not None else None

    def process_record(self, record, dbm):
        r = self.get_record(record, dbm)
        if self.units is not None and record['usUnits'] != self.units:
            converter = weewx.units.StdUnitConverters[self.units]
            r = converter.convertDict(r)
        url = urlparse.urlparse(self.server_url)
        mc = mqtt.Client()
        mc.username_pw_set(url.username, url.password)
        mc.connect(url.hostname, url.port)
        for key in r:
            mc.publish(self.topic + "/" + key, r[key])
        mc.disconnect()
